@@ -4,7 +4,7 @@
 """
 ===============================================================================
 
-    Copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
+    Original code copyright (C) 2009-2018 Rudolf Cardinal (rudolf@pobox.com).
 
     This file is part of cardinal_pythonlib.
 
@@ -21,6 +21,11 @@
     limitations under the License.
 
 ===============================================================================
+
+**Functions to operate with the raw Python database API.**
+
+See https://www.python.org/dev/peps/pep-0249/.
+
 """
 
 from collections import OrderedDict
@@ -31,13 +36,22 @@ from typing import Any, Dict, Generator, List, Optional
 
 def get_fieldnames_from_cursor(cursor) -> List[str]:
     """
-    Get fieldnames from an executed cursor.
+    Get a list of fieldnames from an executed cursor.
     """
     return [i[0] for i in cursor.description]
 
 
 def genrows(cursor, arraysize: int = 1000) -> Generator[List[Any], None, None]:
-    """Generate all rows from a cursor."""
+    """
+    Generate all rows from a cursor.
+
+    Args:
+        cursor: the cursor
+        arraysize: split fetches into chunks of this many records
+
+    Yields:
+        each row
+    """
     # http://code.activestate.com/recipes/137270-use-generators-for-fetching-large-db-record-sets/  # noqa
     while True:
         results = cursor.fetchmany(arraysize)
@@ -48,7 +62,16 @@ def genrows(cursor, arraysize: int = 1000) -> Generator[List[Any], None, None]:
 
 
 def genfirstvalues(cursor, arraysize: int = 1000) -> Generator[Any, None, None]:
-    """Generate the first value in each row."""
+    """
+    Generate the first value in each row.
+
+    Args:
+        cursor: the cursor
+        arraysize: split fetches into chunks of this many records
+
+    Yields:
+        the first value of each row
+    """
     return (row[0] for row in genrows(cursor, arraysize))
 
 
@@ -59,7 +82,17 @@ def fetchallfirstvalues(cursor) -> List[Any]:
 
 def gendicts(cursor, arraysize: int = 1000) -> Generator[Dict[str, Any],
                                                          None, None]:
-    """Generate all rows from a cursor as a list of OrderedDicts."""
+    """
+    Generate all rows from a cursor as :class:`OrderedDict` objects.
+
+    Args:
+        cursor: the cursor
+        arraysize: split fetches into chunks of this many records
+
+    Yields:
+        each row, as an :class:`OrderedDict` whose key are column names
+        and whose values are the row values
+    """
     columns = get_fieldnames_from_cursor(cursor)
     return (
         OrderedDict(zip(columns, row))
@@ -68,7 +101,16 @@ def gendicts(cursor, arraysize: int = 1000) -> Generator[Dict[str, Any],
 
 
 def dictfetchall(cursor) -> List[Dict[str, Any]]:
-    """Return all rows from a cursor as a list of OrderedDicts."""
+    """
+    Return all rows from a cursor as a list of :class:`OrderedDict` objects.
+
+    Args:
+        cursor: the cursor
+
+    Returns:
+        a list (one item per row) of :class:`OrderedDict` objects whose key are
+        column names and whose values are the row values
+    """
     columns = get_fieldnames_from_cursor(cursor)
     return [
         OrderedDict(zip(columns, row))
@@ -78,7 +120,7 @@ def dictfetchall(cursor) -> List[Dict[str, Any]]:
 
 def dictfetchone(cursor) -> Optional[Dict[str, Any]]:
     """
-    Return the next row from a cursor as an OrderedDict, or None
+    Return the next row from a cursor as an :class:`OrderedDict`, or ``None``.
     """
     columns = get_fieldnames_from_cursor(cursor)
     row = cursor.fetchone()
